import os

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand
from plumbum import FG, local


class Command(BaseCommand):
    help = "start prod server"

    def handle(self, *args, **options):
        call_command("collectstatic", "--no-input")
        call_command("migrate")
        p = os.environ["DJANGO_SETTINGS_MODULE"].split(".")[0]
        (
            local.get(".venv/bin/gunicorn")[
                "--bind",
                settings.GUNICORN_BIND,
                f"--workers={settings.GUNICORN_WORKERS}",
                "-k",
                "egg:meinheld#gunicorn_worker",
                "--worker-tmp-dir",
                "/dev/shm",
                f"{p}.base.wsgi:application",
            ]
            & FG
        )



import uuid

from django.test import TestCase
from pdfminer.high_level import extract_text

from tuhls_invoice import models
from tuhls_invoice.services import invoice_to_pdf


class InvoiceTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = uuid.uuid4()

    @staticmethod
    def create_fake_invoice(user, invoice_type, num_products):
        invoice = models.Invoice.objects.create(
            user=user,
            type=invoice_type,
            city="some_city",
            zip_code="0815",
            state="some_state",
        )
        for x in range(num_products):
            models.InvoiceDetail.objects.create(
                invoice=invoice,
                name=f"product {x}",
                description=f"nice product {x}",
                amount=5,
            )
        return invoice

    def test_invoices(self):
        invoice = self.create_fake_invoice(self.user, models.Invoice.Types.INVOICE, 5)
        self.assertEqual(25, invoice.total())
        self.assertEqual("i-000001", invoice.get_invoice_number())
        self.assertEqual("invoice", invoice.get_type_name())

        models.InvoiceDetail.objects.create(
            invoice=invoice,
            name="credit",
            description="nice credit",
            amount=-15,
        )
        self.assertEqual(10, invoice.total())

        for i in [
            [self.user, models.Invoice.Types.ESTIMATE, 3, 15, "e-000001", "estimate"],
            [self.user, models.Invoice.Types.ESTIMATE, 7, 35, "e-000002", "estimate"],
            [self.user, models.Invoice.Types.INVOICE, 1, 5, "i-000002", "invoice"],
            [self.user, models.Invoice.Types.ESTIMATE, 2, 10, "e-000003", "estimate"],
        ]:
            invoice = self.create_fake_invoice(i[0], i[1], i[2])
            self.assertEqual(i[3], invoice.total())
            self.assertEqual(i[4], invoice.get_invoice_number())
            self.assertEqual(i[5], invoice.get_type_name())

    def test_pdf(self):
        invoice = self.create_fake_invoice(self.user, models.Invoice.Types.INVOICE, 5)
        txt = extract_text(invoice_to_pdf(invoice))
        self.assertIn("Total", txt)
        self.assertIn("$ 25.00", txt)
        self.assertIn("INVOICE", txt)



from django.conf import settings
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver

from tuhls_core.models import CustomModel


class Invoice(CustomModel):
    class Types(models.TextChoices):
        INVOICE = "i", "invoice"
        ESTIMATE = "e", "estimate"
        CREDIT_NOTE = "cn", "credit note"

    user = models.UUIDField()
    type = models.CharField(max_length=2, choices=Types.choices, blank=True, null=True)
    number = models.IntegerField(blank=True, null=True)

    name = models.CharField(max_length=63, blank=True, null=True)
    address1 = models.CharField(max_length=63, blank=True, null=True)
    address2 = models.CharField(max_length=63, blank=True, null=True)
    zip_code = models.CharField(max_length=63, blank=True, null=True)
    city = models.CharField(max_length=63, blank=True, null=True)
    country = models.CharField(max_length=63, blank=True, null=True)
    state = models.CharField(max_length=63, blank=True, null=True)
    company = models.CharField(max_length=63, blank=True, null=True)
    vat = models.CharField(max_length=63, blank=True, null=True)

    payment_reference = models.CharField(max_length=63, blank=True, null=True)
    payment_provider = models.CharField(max_length=63, blank=True, null=True)
    is_paid = models.BooleanField(default=False)

    def total(self):
        t = 0
        for detail in self.details.all():
            t = t + detail.amount
        return t

    def get_type_name(self):
        return dict(self.Types.choices)[self.type]

    def get_invoice_number(self):
        if self.number:
            return f"{self.type}-{self.number:06d}"
        else:
            """ """

    def get_file_name(self):
        return f"{settings.SAAS_SHORT_NAME}-{self.get_type_name()}-{self.get_invoice_number()}.pdf"


@receiver(post_save, sender=Invoice)
def find_next_number(sender: Invoice, instance: Invoice, created: bool, **kwargs):
    if not instance.number:
        last_invoice = (
            sender.objects.filter(type=instance.type, number__gt=0)
            .order_by("-number")
            .first()
        )
        if last_invoice and last_invoice.number and last_invoice.number > 0:
            instance.number = last_invoice.number + 1
        else:
            instance.number = 1
        instance.save()


class InvoiceDetail(CustomModel):
    invoice = models.ForeignKey(
        Invoice, on_delete=models.CASCADE, related_name="details"
    )

    name = models.CharField(max_length=63)
    description = models.CharField(max_length=255, blank=True, null=True)
    amount = models.DecimalField(max_digits=8, decimal_places=2)



DEBUG = True
SECRET_KEY = "testkeytestkey"
INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "tuhls_core",
    "tuhls_invoice",
    "example",
]
AUTH_USER_MODEL = "example.User"
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}



import copy
import os
from io import BytesIO

from reportlab.lib import colors
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import mm
from reportlab.pdfgen import canvas
from reportlab.platypus import Paragraph, SimpleDocTemplate, Table
from svglib.svglib import svg2rlg

# always available fonts: Courier, Courier-Bold, Courier-BoldOblique, Courier-Oblique, Helvetica, Helvetica-Bold,
# Helvetica-BoldOblique, Helvetica-Oblique, Symbol, Times-Bold, Times-BoldItalic, Times-Italic, Times-Roman,
# ZapfDingbats
# default reportlab font: Helvetica

gdata = {}


class PageCanvas(canvas.Canvas):
    def __init__(self, *args, **kwargs):
        canvas.Canvas.__init__(self, *args, **kwargs)
        self._saved_page_states = []

    def showPage(self):
        self._saved_page_states.append(dict(self.__dict__))
        self._startPage()

    def save(self):
        for state in self._saved_page_states:
            self.__dict__.update(state)
            self.before_save()
            canvas.Canvas.showPage(self)
        canvas.Canvas.save(self)

    def before_save(self):
        num_pages = len(self._saved_page_states)

        self.setFont("Helvetica", 8)
        self.setFillColorCMYK(0, 0, 0, 0.75)

        self.drawRightString(
            A4[0] - (25 * mm), 15 * mm, f"Page {self._pageNumber}/{num_pages}"
        )

        self.drawString(
            25 * mm,
            18 * mm,
            gdata["footer"][0],
        )

        self.drawString(
            25 * mm,
            15 * mm,
            gdata["footer"][1],
        )


def get_svg(path, sx=1, sy=1):
    svg = svg2rlg(path)
    svg.width, svg.height = svg.minWidth() * sx, svg.height * sy
    svg.scale(sx, sy)
    return svg


def address_table(data, style):
    receiver = [
        x
        for x in [
            data["company"],
            data["name"],
            " ",
            data["address1"],
            data["address2"],
            data["city"] + ", " + data["state"] + " " + data["zip_code"],
            data["country"],
        ]
        if x is not None and x != ""
    ]

    t = Table(
        [
            [Paragraph("<br/>".join(receiver), style)],
        ],
        colWidths=["100%"],
        style=[],
    )
    return t


def create(d):
    global gdata
    gdata = d
    buffer = BytesIO()
    styles = getSampleStyleSheet()
    style_normal = copy.copy(styles["BodyText"])

    doc = SimpleDocTemplate(
        buffer,
        pagesize=A4,
        title=d["type"],
        creator=d["creator"],
        pageCompression=1,
        topMargin=10 * mm,
        leftMargin=25 * mm,
        rightMargin=25 * mm,
        bottomMargin=25 * mm,
    )

    color_gray_light = colors.CMYKColor(0, 0, 0, 0.1)
    color_gray = colors.CMYKColor(0, 0, 0, 0.75)

    style_item_header = copy.copy(styles["BodyText"])

    style_normal_right = copy.copy(styles["BodyText"])
    style_normal_right.alignment = 2

    style_document_header = copy.copy(styles["BodyText"])
    style_document_header.fontSize = 18
    style_document_header.alignment = 1

    style_item_description = copy.copy(styles["BodyText"])
    style_item_description.fontSize = 8
    style_item_description.textColor = color_gray

    flow = []
    flow.append(
        Table(
            [
                [
                    "",
                    get_svg(
                        os.path.join(
                            os.path.dirname(__file__), "data/invoice_logo.svg"
                        ),
                        3,
                        3,
                    ),
                ],
            ],
            colWidths=["50%", "50%"],
            style=[
                ("ALIGN", (1, 0), (1, 0), "RIGHT"),
                ("BOTTOMPADDING", (0, 0), (1, 0), 20 * mm),
            ],
        )
    )
    flow.append(address_table(d["to"], style_normal))
    flow.append(
        Table(
            [
                ["", d["type"].upper()],
                ["", "Date"],
                ["", d["date"]],
            ],
            colWidths=["50%", "60%"],
            style=[
                ("ALIGN", (1, 0), (1, 2), "RIGHT"),
                ("FONTSIZE", (1, 0), (1, 0), 18),
                ("TEXTCOLOR", (1, 1), (1, 1), color_gray),
                ("BOTTOMPADDING", (0, 0), (1, 0), 10 * mm),
                ("BOTTOMPADDING", (0, 2), (1, 2), 10 * mm),
            ],
        )
    )

    style = [
        ("BACKGROUND", (0, 0), (2, 0), color_gray_light),
        ("TEXTCOLOR", (0, 0), (2, 0), color_gray),
    ]

    amount_sum = 0
    num_rows = 1
    data = [["Item", "", "Total"]]
    for item in d["items"]:
        data.append(
            [
                (
                    Paragraph(item["name"], style_item_header),
                    Paragraph(item["description"], style_item_description),
                ),
                "",
                d["currency_symbol"]
                + " "
                + (str(item["amount"]) if item["amount"] else "0"),
            ]
        )
        amount_sum += item["amount"]

        style.append(("LINEBELOW", (0, num_rows), (2, num_rows), 1, color_gray_light))
        num_rows += 1

    data.append(
        ["", "Total", d["currency_symbol"] + " " + "%.2f" % round(amount_sum, 2)]
    )

    num_rows += 2
    style.append(("ALIGN", (0, 0), (0, num_rows), "LEFT"))
    style.append(("ALIGN", (1, 0), (2, num_rows), "RIGHT"))
    style.append(("TEXTCOLOR", (0, num_rows - 2), (1, num_rows), color_gray))
    flow.append(Table(data, colWidths=["70%", "15%", "15%"], repeatRows=1, style=style))

    doc.build(flow, canvasmaker=PageCanvas)
    buffer.seek(0)
    return buffer



import json

from tuhls_core.utils import request_to_ip

from .models import Request


class RequestLoggerMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        if (
            request.user
            and hasattr(request.user, "gid")
            and hasattr(request.resolver_match, "url_name")
            and request.resolver_match.url_name in ["number_check"]
        ):
            Request.objects.create(
                user=request.user.gid,
                method=request.method,
                status=response.status_code,
                ip=request_to_ip(request),
                request_data=json.dumps(request.GET.__getstate__(), indent=4),
                response_data=json.dumps(response.data, indent=4),
            )
            # print(f"Request URL: {request.get_raw_uri()}")
            # print(f"Request HEADERS: {request.headers}")
            # print(f"Request POST data: {request.POST}")
            # print(f"Request FILES data: {request.FILES}")
            #
            # try:
            #     print(f"Response MEDIA_TYPE: {response.accepted_media_type}")
            #     print(f"Response _MEDIA_TYPE: {response.charset}")
            # except:
            #     print(f"Response MEDIA_TYPE: {response.charset}")
            #     try:
            #         print(f"Response DATA: {response.content}")
            #     except:
            #         print(f"Response DATA: {response.streaming_content}")

        return response



from django.contrib.auth import get_user_model
from django.test import TestCase


class UsersManagersTests(TestCase):
    def test_user_fail_without_params(self):
        with self.assertRaises(TypeError):
            get_user_model().objects.create_user()

    def test_user_fail_with_empty_email(self):
        with self.assertRaises(ValueError):
            get_user_model().objects.create_user(email="", password="12341234")

    def test_create_user(self):
        user = get_user_model().objects.create_user(
            email=" user@email.com ", password="user"
        )
        self.assertEqual(user.email, "user@email.com")
        self.assertTrue(user.is_active)
        self.assertFalse(user.is_staff)
        self.assertFalse(user.is_superuser)

    def test_create_superuser(self):
        admin_user = get_user_model().objects.create_superuser(
            "admin@email.com  ", "admin"
        )
        self.assertEqual(admin_user.email, "admin@email.com")
        self.assertTrue(admin_user.is_active)
        self.assertTrue(admin_user.is_staff)
        self.assertTrue(admin_user.is_superuser)

    def test_str(self):
        test_user = get_user_model().objects.create_user(
            email="test@email.com", password="test"
        )
        self.assertEqual(str(test_user), "test@email.com")



from django.urls import path

from . import views

urlpatterns = [
    path("overview/", views.invoice_overview, name="invoice_overview"),
    path("invoice_pdf/<uuid:invoice_gid>", views.invoice_pdf, name="invoice_pdf"),
    path("receipt_pdf/<uuid:invoice_gid>", views.receipt_pdf, name="receipt_pdf"),
]



from django import template
from django.utils.safestring import mark_safe
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name

register = template.Library()


@register.simple_tag()
def codeformat(code, format):
    lexer = get_lexer_by_name(format, stripall=True)
    formatter = HtmlFormatter(linenos=True, cssclass="source")
    return mark_safe(highlight(code, lexer, formatter))



from django.contrib.auth.decorators import login_required
from django.shortcuts import render

from tuhls_core.utils import AuthenticatedHttpRequest

from .models import Request


@login_required()
def logs(request: AuthenticatedHttpRequest):
    logs = Request.objects.filter(user=request.user.gid).order_by("-created_at")

    return render(
        request,
        "dashboard/logs.html",
        {"logs": logs},
    )



from django.core.management.base import BaseCommand
from django.db import connection


class Command(BaseCommand):
    help = "drop tables"

    def handle(self, *args, **options):
        with connection.cursor() as cursor:
            cursor.execute(
                """
                SELECT table_name
                FROM information_schema.tables
                WHERE table_type='BASE TABLE'
                AND table_schema='public';
            """
            )
            tables = cursor.fetchall()

        with connection.cursor() as cursor:
            for table in tables:
                if table[0] in ["spatial_ref_sys"]:
                    continue
                self.stdout.write(self.style.SUCCESS(f"dropping {table[0]}"))
                cursor.execute(f"drop table {table[0]} CASCADE;")

        self.stdout.write(self.style.SUCCESS("Successfully dropped tables"))



from setuptools import find_packages, setup

import versioneer

setup(
    name="tuhls_core",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=find_packages(),
    package_data={
        "tuhls_design": ("static/country_flags/svg/*.svg", "static/css/dist/style.css"),
    },
    install_requires=["django"],
    license_files=("LICENSE",),
    url="https://gitlab.com/tuhls/tuhls_core",
    author="Herbert Rusznak (tlb)",
    author_email="herbert.rusznak@gmail.com",
    license="MIT",
    python_requires=">=3.8",
)



from django.contrib.auth.decorators import login_required
from django.http import FileResponse
from django.shortcuts import redirect, render

from tuhls_core.utils import AuthenticatedHttpRequest

from .models import Invoice
from .pdf import create
from .services import invoice_to_pdf


@login_required()
def invoice_overview(request: AuthenticatedHttpRequest):
    invoices_due = Invoice.objects.filter(
        user=request.user.gid, type=Invoice.Types.INVOICE, is_paid=False
    ).order_by("-created_at")
    invoices_paid = Invoice.objects.filter(
        user=request.user.gid, type=Invoice.Types.INVOICE, is_paid=True
    ).order_by("-created_at")
    return render(
        request,
        "dashboard/invoice.html",
        {"invoices_due": invoices_due, "invoices_paid": invoices_paid},
    )


def invoice_pdf(request: AuthenticatedHttpRequest, invoice_gid):
    try:
        invoice = Invoice.objects.get(gid=invoice_gid)
    except Invoice.DoesNotExist:
        return redirect("invoice_overview")

    if str(invoice.user) != str(request.user.gid):
        return redirect("invoice_overview")

    return FileResponse(
        invoice_to_pdf(invoice),
        as_attachment=False,
        filename=invoice.get_file_name(),
        content_type="application/pdf",
    )


def receipt_pdf(request: AuthenticatedHttpRequest, invoice_gid):
    try:
        invoice = Invoice.objects.get(gid=invoice_gid)
    except Invoice.DoesNotExist:
        return redirect("invoice_overview")

    if str(invoice.user) != str(request.user.gid):
        return redirect("invoice_overview")

    if not invoice.is_paid:
        return redirect("invoice_overview")

    b = create(
        {
            "to": {
                "name": invoice.name,
                "company": invoice.company,
                "address1": invoice.address1,
                "address2": invoice.address2,
                "zip_code": invoice.zip_code,
                "state": invoice.state,
                "city": invoice.city,
                "country": invoice.country,
            },
            "type": "Receipt",
            "creator": "Bilcom GesmbH.",
            "date": invoice.created_at.strftime("%Y-%m-%d %H:%M"),
            "currency_symbol": "$",
            "footer": [
                "Bankverbindung: Erste Bank, IBAN: AT96 2011 1822 6402 1500, BIC GIBAATWWXXX",
                "Bilcom GesmbH. Universitätstraße 4/4, 1090 Wien, office@bilcom.at, +43 1 641 99 85",
            ],
            "items": [
                {
                    "name": x.name,
                    "description": x.description,
                    "amount": x.amount,
                }
                for x in invoice.details.all()
            ],
        }
    )
    return FileResponse(
        b,
        as_attachment=False,
        filename=invoice.get_file_name(),
        content_type="application/pdf",
    )



import importlib
import os

from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "load dev data"

    def handle(self, *args, **options):
        call_command("drop_tables")
        call_command("makemigrations")
        call_command("migrate")
        p = os.environ["DJANGO_SETTINGS_MODULE"].split(".")[0]
        fixtures = importlib.import_module(f"{p}.fixtures")
        fixtures.create_all()
        self.stdout.write(self.style.SUCCESS("Successfully created dev data"))



from django.core.management.commands.test import Command as BaseCommand


class Command(BaseCommand):
    def handle(self, *test_labels, **options):
        # Wrap Django's built-in test command to always delete the database if it exists
        options["interactive"] = False
        return super().handle(*test_labels, **options)
